/*
 * _1284_DramTester.c
 *
 * Created: 11/24/2014 11:38:47 AM
 *  Author: lbay
 */ 


#define F_CPU 16000000    // AVR clock frequency in Hz, used by util/delay.h
#include <avr/io.h>
#include <util/delay.h>

#define LOOPS	2

// PORTB Defines
#define GREEN	0		
#define RED		1
#define BLUE	2

// PORTD Defines 
#define DIN		3	// DRAM Data IN 	
#define DOUT	4	// DRAM	Data Out
#define CAS		5	// DRAM CAS Line
#define RAS		6	// DRAM	RAS Line
#define WE		7	// DRAM WE Line

#define cbi_b(bit) PORTB &= ~(_BV(bit))
#define sbi_b(bit) PORTB |= _BV(bit)

#define cbi_d(bit) PORTD &= ~(_BV(bit))
#define sbi_d(bit) PORTD |= _BV(bit)

#define _NOP() do { __asm__ __volatile__ ("nop"); } while (0) 
	

void	fillZero();
void	fillOne();
void	fillZeroAlternateOne();
void	fillOneAlternateZero();

void	initDram();


int main(void)
	{
	int	count = 0, i;
	
	
	DDRA = 0xFF;			// DRAM Low Address
	DDRC = 0x01;			// DRAM High Address
	DDRB = 0x07;			// LEDs on PORTB
	DDRD = 0xE8;			// DRAM Control and Data lines
	
	cbi_b(GREEN);			// Clear all LED's
	cbi_b(RED);
	cbi_b(BLUE);
	
	cbi_d(DIN);				// Clear in data line
	sbi_d(CAS);				// Set control lines HIGH
	sbi_d(RAS);
	sbi_d(WE);
	
	initDram();
	
	sbi_b(GREEN);
	
	
    while(1)
		{
		fillZero();
		fillOne();	
		fillZeroAlternateOne();
		fillOneAlternateZero();	
		
		sbi_b(BLUE);	// Set BLUE LED if all tests pass
		
		++count;
		
		for(i = count ; i ; i--)	// Blink RED LED to show how many good loops we have had
			{
			sbi_b(RED);
			_delay_ms(500);	
			cbi_b(RED);
			_delay_ms(500);
			}
		}
	}
	

// TEST ONE - All ZEROS
void	fillZero()
	{
	int		row, col, miss = 0, loop;
	char	*rptr, *cptr;
	
	
	rptr = (char *) &row;		// Setup pointer to high row byte
	++rptr;
	
	cptr = (char *) &col;		// Setup pointer to high column byte
	++cptr;
	
	// First write all Zeros to DRAM
		
	cbi_d(DIN);			// Set DRAM Data Line to 0
	
	for(col = 0 ; col < 512 ; col++)
		{
		for(row = 0 ; row < 512 ; row++)
			{
			PORTA = row;	// Write Row Address
			PORTC = *rptr;
		//_NOP();
			cbi_d(RAS);		// Set RAS Low
		//_NOP();
			cbi_d(WE);		// Set WE Low
		//_NOP();	
			PORTA = col;	// Write Column Address
			PORTC = *cptr;	
		//_NOP();	
			cbi_d(CAS);		// Set CAS Low
		//_NOP();	
			sbi_d(WE);		// Set WE High
		//_NOP();
			sbi_d(CAS);		// Set CAS High
		//_NOP();
			sbi_d(RAS);		// Set RAS High	
		//_NOP();		
		
			_delay_us(5);
			}
		}
		

	// Read DRAM BACK and make sure every bit is zero
	
	sbi_d(DIN);			// Set DRAM Data Line to 1 (I just do this so I can see the cycle on my logic analyzer.)
	
	for(loop = LOOPS ; loop ; loop--)
		{
		for(col = 0 ; col < 512 ; col++)
			{
			for(row = 0 ; row < 512 ; row++)
				{
				PORTA = row;	// Write Row Address
				PORTC = *rptr;
			//_NOP();	
				cbi_d(RAS);		// Set RAS Low
			//_NOP();	
				PORTA = col;	// Write Column Address
				PORTC = *cptr;
			//_NOP();	
				cbi_d(CAS);		// Set CAS Low
				_NOP();	
				_NOP();
				_NOP();
				_NOP();
				_NOP();
				_NOP();
				_NOP();
				_NOP();
				
				if(PIND & 0x10)
					miss = 1;
			//_NOP();		
				sbi_d(CAS);		// Set CAS High
			//_NOP();
				sbi_d(RAS);		// Set RAS High
			//_NOP();
		
				_delay_us(4);
				_NOP();
				_NOP();
				_NOP();
				_NOP();
				_NOP();
				_NOP();
				_NOP();
				_NOP();		
				}
			}
	
		if(miss)
			break;
		}
			
	if(miss)
		{
		cbi_b(BLUE);
		cbi_b(GREEN);	
		sbi_b(RED);	
		
		while(1)
			;
		}	
			
	}
	
	
	
// TEST TWO - All ONEs
void	fillOne()
{
	int		row, col, miss = 0, loop;
	char	*rptr, *cptr;
	
	
	rptr = (char *) &row;		// Setup pointer to high row byte
	++rptr;
	
	cptr = (char *) &col;		// Setup pointer to high column byte
	++cptr;
	
	// First write all Ones to DRAM
	
	sbi_d(DIN);			// Set DRAM Data Line to 1
	
	for(col = 0 ; col < 512 ; col++)
		{
		for(row = 0 ; row < 512 ; row++)
			{
			PORTA = row;	// Write Row Address
			PORTC = *rptr;
		_NOP();
			cbi_d(RAS);		// Set RAS Low
		_NOP();
			cbi_d(WE);		// Set WE Low
		_NOP();
			PORTA = col;	// Write Column Address
			PORTC = *cptr;
		_NOP();
			cbi_d(CAS);		// Set CAS Low
		_NOP();
			sbi_d(WE);		// Set WE High
		_NOP();
			sbi_d(CAS);		// Set CAS High
		_NOP();
			sbi_d(RAS);		// Set RAS High
		_NOP();
		
		_delay_us(4);
		}
	}
	

	// Read DRAM BACK and make sure every bit is zero
	
	cbi_d(DIN);			// Set DRAM Data Line to 0 (I just do this so I can see the cycle on my logic analyzer.)
	
	for(loop = LOOPS ; loop ; loop--)
		{	
		for(col = 0 ; col < 512 ; col++)
			{
			for(row = 0 ; row < 512 ; row++)
				{
				PORTA = row;	// Write Row Address
				PORTC = *rptr;
			_NOP();
				cbi_d(RAS);		// Set RAS Low
			_NOP();
				PORTA = col;	// Write Column Address
				PORTC = *cptr;
			_NOP();
				cbi_d(CAS);		// Set CAS Low
			
			_NOP();	
			_NOP();
				if(!(PIND & 0x10))
					miss = 1;
			_NOP();
				sbi_d(CAS);		// Set CAS High
			_NOP();
				sbi_d(RAS);		// Set RAS High
			_NOP();
		
			_delay_us(3);
			}
		}
	if(miss)
		break;	
	}
	
	if(miss)
		{
		cbi_b(BLUE);
		cbi_b(GREEN);
		sbi_b(RED);
		
		while(1)
			;
		}
	
}
	
	
// TEST THREE - Alternate Zero and One
void	fillZeroAlternateOne()
{
	int		row, col, miss = 0, i = 0, loop;
	char	*rptr, *cptr;
	
	
	rptr = (char *) &row;		// Setup pointer to high row byte
	++rptr;
	
	cptr = (char *) &col;		// Setup pointer to high column byte
	++cptr;
	
	// Alternate Zeros and 0nes
	
	if(i)
		{
	     i = 0;
		 sbi_d(DIN);			// Set DRAM Data Line to 1
		}
	else
		{
	     i = 1;
		 cbi_d(DIN);			// Set DRAM Data Line to 0
		 }
		 
	for(col = 0 ; col < 512 ; col++)
	{
		for(row = 0 ; row < 512 ; row++)
		{
			PORTA = row;	// Write Row Address
			PORTC = *rptr;
			_NOP();
			cbi_d(RAS);		// Set RAS Low
			_NOP();
			cbi_d(WE);		// Set WE Low
			_NOP();
			PORTA = col;	// Write Column Address
			PORTC = *cptr;
			_NOP();
			cbi_d(CAS);		// Set CAS Low
			_NOP();
			sbi_d(WE);		// Set WE High
			_NOP();
			sbi_d(CAS);		// Set CAS High
			_NOP();
			sbi_d(RAS);		// Set RAS High
			_NOP();
			
			_delay_us(4);
		}
	}
	

	// Read DRAM BACK and make sure every bit alternates zero and then one
	
	cbi_d(DIN);			// Set DRAM Data Line to 0 (I just do this so I can see the cycle on my logic analyzer.)

	for(loop = LOOPS ; loop ; loop--)
		{
		i = 0;	
		for(col = 0 ; col < 512 ; col++)
			{
			for(row = 0 ; row < 512 ; row++)
				{
				PORTA = row;	// Write Row Address
				PORTC = *rptr;
				_NOP();
				cbi_d(RAS);		// Set RAS Low
				_NOP();
				PORTA = col;	// Write Column Address
				PORTC = *cptr;
				_NOP();
				cbi_d(CAS);		// Set CAS Low
			
				_NOP();
				_NOP();
			
				if(i)
					{
					if(!(PIND & 0x10))
						miss = 1;				
					}
				else
					{
					if((PIND & 0x10))
						miss = 1;	
					}
			
			
				_NOP();
				sbi_d(CAS);		// Set CAS High
				_NOP();
				sbi_d(RAS);		// Set RAS High
				_NOP();
			
				_delay_us(3);
			}
		}
	
	if(miss)
		break;
	}
	
	if(miss)
		{
		cbi_b(BLUE);
		cbi_b(GREEN);
		sbi_b(RED);
		
		while(1)
			;
		}
	
}	
	
	
// TEST FOUR - Alternate  One and Zero
	void	fillOneAlternateZero()
	{
		int		row, col, miss = 0, i = 1, loop;
		char	*rptr, *cptr;
		
		
		rptr = (char *) &row;		// Setup pointer to high row byte
		++rptr;
		
		cptr = (char *) &col;		// Setup pointer to high column byte
		++cptr;
		
		// Alternate Ones and Zeros
		
		if(i)
		{
			i = 0;
			sbi_d(DIN);			// Set DRAM Data Line to 1
		}
		else
		{
			i = 1;
			cbi_d(DIN);			// Set DRAM Data Line to 0
		}
		
		for(col = 0 ; col < 512 ; col++)
		{
			for(row = 0 ; row < 512 ; row++)
			{
				PORTA = row;	// Write Row Address
				PORTC = *rptr;
				_NOP();
				cbi_d(RAS);		// Set RAS Low
				_NOP();
				cbi_d(WE);		// Set WE Low
				_NOP();
				PORTA = col;	// Write Column Address
				PORTC = *cptr;
				_NOP();
				cbi_d(CAS);		// Set CAS Low
				_NOP();
				sbi_d(WE);		// Set WE High
				_NOP();
				sbi_d(CAS);		// Set CAS High
				_NOP();
				sbi_d(RAS);		// Set RAS High
				_NOP();
				
				_delay_us(4);
			}
		}
		

	// Read DRAM BACK and make sure every bit alternates one and then zero 
		
	cbi_d(DIN);			// Set DRAM Data Line to 0 (I just do this so I can see the cycle on my logic analyzer.)
		
	for(loop = LOOPS ; loop ; loop--)
		{		
		i = 1;
		
		for(col = 0 ; col < 512 ; col++)
			{
			for(row = 0 ; row < 512 ; row++)
				{
				PORTA = row;	// Write Row Address
				PORTC = *rptr;
				_NOP();
				cbi_d(RAS);		// Set RAS Low
				_NOP();
				PORTA = col;	// Write Column Address
				PORTC = *cptr;
				_NOP();
				cbi_d(CAS);		// Set CAS Low
				
				_NOP();
				_NOP();
				
				if(i)
				{
					if(!(PIND & 0x10))
					miss = 1;
				}
				else
				{
					if((PIND & 0x10))
					miss = 1;
				}
				
				
				_NOP();
				sbi_d(CAS);		// Set CAS High
				_NOP();
				sbi_d(RAS);		// Set RAS High
				_NOP();
				
				_delay_us(3);
			}
		}
		if(miss)
			break;
		}
		
		if(miss)
		{
			cbi_b(BLUE);
			cbi_b(GREEN);
			sbi_b(RED);
			
			while(1)
			;
		}
		
	}
	
	
void	initDram()
		{
		int	i;
		
		
		_delay_us(250);         // Initial DRAM startup delay 250us		

		PORTA = 0x00;
		PORTC = 0x00;
				
		for(i = 0 ; i < 8 ; i++)
			{
			cbi_d(RAS);
			sbi_d(RAS);	
			}
		
		}
	
	